@@ -3,8 +3,13 @@
 #' \href{https://gist.github.com/kerryrodden/7090426}{Sequences sunburst} diagrams provide
 #'    an interactive method of exploring sequence data, such as website navigation paths.
 #'
-#' @param csvdata data in csv source,target form
-#' @param jsondata data in nested d3 JSON hierarchy with `{name:...,  children:[];}`
+#' @param data data in csv source,target form or in
+#'          nested d3 JSON hierarchy with `{name:...,  children:[];}`.  \code{csvdata}
+#'          and \code{jsondata} arguments are now deprecated in favor of this single
+#'          \code{data} argument.  \code{list}, \code{character},
+#'          or \code{connection} data will be assumed to be \code{JSON}.
+#'          \code{data.frame} data will be assumed to be \code{csvdata} and converted
+#'          to \code{JSON} by \code{sunburstR:::csv_to_hier()}.
 #' @param legendOrder string vector if you would like to manually order the legend.
 #'          If legendOrder is not provided, then the legend will be in the descending
 #'          order of the top level hierarchy.
@@ -25,22 +30,28 @@
 #'          breadcrumbs widths based on text length.
 #' @param sortFunction \code{\link[htmlwidgets]{JS}} function to sort the slices.
 #'          The default sort is by size.
+#' @param withD3 \code{logical} to include d3 dependency from \code{d3r}.  As of
+#'          version 1.0, sunburst uses a standalone JavaScript build and will
+#'          not include the entire d3 in the global/window namespace.  To include
+#'          d3.js in this way, use \code{withD3=TRUE}.
 #' @param height,width  height and width of sunburst htmlwidget containing div
 #'          specified in any valid \code{CSS} size unit.
 #' @param elementId string id as a valid \code{CSS} element id.
 #' @param sizingPolicy see \code{\link[htmlwidgets]{sizingPolicy}}.
+#' @param csvdata \code{deprecated} use data argument instead; data in csv source,target form
+#' @param jsondata \code{deprecated} use data argument instead; data in nested d3 JSON hierarchy with `{name:...,  children:[];}`
 #'
 #' @example inst/examples/example_replicate.R
 #' @example inst/examples/example_ngram.R
 #' @example inst/examples/example_treemap.R
 #' @example inst/examples/example_calendar.R
+#' @example inst/examples/example_sort.R
 #'
 #' @import htmlwidgets
 #'
 #' @export
 sunburst <- function(
-  csvdata = NULL
-  , jsondata = NULL
+  data = NULL
   , legendOrder = NULL
   , colors = NULL
   , valueField = "size"
@@ -50,14 +61,45 @@ sunburst <- function(
   , breadcrumb = list()
   , legend = list()
   , sortFunction = NULL
+  , withD3 = FALSE
   , width = NULL
   , height = NULL
   , elementId = NULL
   , sizingPolicy = NULL
+  , csvdata = NULL
+  , jsondata = NULL
 ) {
 
-  if(is.null(csvdata) && is.null(jsondata)) stop("please provide either csvdata or jsondata",call.=FALSE)
-  if(!is.null(csvdata) && !is.null(jsondata)) warning("both csv and json provided; will use csvdata",call.=FALSE)
+  if(is.null(data) && is.null(csvdata) && is.null(jsondata)) stop("please provide data",call.=FALSE)
+  if(!is.null(csvdata) || !is.null(jsondata)) {
+    warning("The csvdata and jsondata arguments have been deprecated in favor of a single data argument", call.=FALSE)
+  }
+  if(is.null(data) && !is.null(csvdata)) data <- csvdata
+  if(is.null(data) && !is.null(jsondata)) data <- jsondata
+
+  # accept JSON string as data
+  if( inherits(data,c("character","connection")) ){
+    data = jsonlite::toJSON(
+      jsonlite::fromJSON( data )
+      , auto_unbox = TRUE
+      , dataframe = "rows"
+    )
+  }
+  # accept list as data
+  if( inherits(data, "list") )  {
+    data = jsonlite::toJSON(
+      data
+      , auto_unbox = TRUE
+      , dataframe = "rows"
+    )
+  }
+  # accept data.frame as data
+  #  and convert to JSON with csv_to_hier
+  #  this conversion should be more robust than
+  #  the JavaScript converter
+  if( inherits(data, "data.frame") )  {
+    data = csv_to_hier(data)
+  }
 
   if(!is.null(explanation) && !inherits(explanation,"JS_EVAL")){
     explanation = htmlwidgets::JS(explanation)
@@ -65,8 +107,7 @@ sunburst <- function(
 
   # forward options using x
   x = list(
-    csvdata = csvdata
-    ,jsondata = jsondata
+    data = data
     ,options = list(
       legendOrder = legendOrder
       ,colors = colors
@@ -84,6 +125,12 @@ sunburst <- function(
     sizingPolicy <- htmlwidgets::sizingPolicy(browser.fill=TRUE)
   }
 
+  # include d3 if withD3 is TRUE
+  dep <- NULL
+  if(withD3 == TRUE) {
+    dep <- d3r::d3_dep_v4()
+  }
+
   # create widget
   htmlwidgets::createWidget(
     name = 'sunburst',
@@ -92,7 +139,8 @@ sunburst <- function(
     height = height,
     package = 'sunburstR',
     elementId = elementId,
-    sizingPolicy
+    sizingPolicy = sizingPolicy,
+    dependencies = dep
   )
 }
 
@@ -128,25 +176,22 @@ renderSunburst <- function(expr, env = parent.frame(), quoted = FALSE) {
 #' @import htmltools
 #' @keywords internal
 sunburst_html <- function(id, style, class, ...){
-  attachDependencies(
-    tagList(
-      tags$div( id = id, class = class, style = style, style="position:relative;"
-        ,tags$div(
-          tags$div(class = "sunburst-main"
-            , tags$div( class = "sunburst-sequence" )
-            , tags$div( class = "sunburst-chart"
-                ,tags$div( class = "sunburst-explanation", style = "visibility:hidden;"
-           #       ,tags$span( class = "sunburst-percentage")
-                )
-            )
-          )
-          ,tags$div(class = "sunburst-sidebar"
-            , tags$input( type = "checkbox", class = "sunburst-togglelegend", "Legend" )
-            , tags$div( class = "sunburst-legend", style = "visibility:hidden;" )
+  tagList(
+    tags$div( id = id, class = class, style = style, style="position:relative;"
+      ,tags$div(
+        tags$div(class = "sunburst-main"
+          , tags$div( class = "sunburst-sequence" )
+          , tags$div( class = "sunburst-chart"
+              ,tags$div( class = "sunburst-explanation", style = "visibility:hidden;"
+         #       ,tags$span( class = "sunburst-percentage")
+              )
           )
         )
+        ,tags$div(class = "sunburst-sidebar"
+          , tags$input( type = "checkbox", class = "sunburst-togglelegend", "Legend" )
+          , tags$div( class = "sunburst-legend", style = "visibility:hidden;" )
+        )
       )
-    ),
-    d3r::d3_dep_v3()
+    )
   )
 }
